/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: standard File operations
//-----------------------------------------------------------------------------

#include "StdFileOperations.h"
#include <malloc.h>

//-----------------------------------------------------------------------------
// Name: DoesFileExists
// Object: check if file exists
// Parameters :
//     in : TCHAR* FullPath : full path
// Return : TRUE if file exists
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::DoesFileExists(TCHAR* FullPath)
{
    BOOL bFileExists=FALSE;

    HANDLE hFile=CreateFile(FullPath,
                            FILE_READ_ATTRIBUTES, 
                            FILE_SHARE_READ|FILE_SHARE_WRITE,
                            NULL,
                            OPEN_EXISTING, 
                            0, 
                            NULL);
    if (hFile != INVALID_HANDLE_VALUE)
    {
        bFileExists=TRUE;
        CloseHandle(hFile);
    }
    return bFileExists;
}

//-----------------------------------------------------------------------------
// Name: GetFileSize
// Object: Get file size 
// Parameters :
//     in : TCHAR* FullPath : full path
//     our : DWORD* pFileSize : file size
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetFileSize(TCHAR* FullPath,OUT DWORD* pFileSize)
{
    BOOL bRet=TRUE;
    // Open the file 
    HANDLE hFile=CreateFile(FullPath,GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
        OPEN_EXISTING, 0, NULL);

    if( hFile == INVALID_HANDLE_VALUE ) 
        return FALSE; 
    // Obtain the size of the file 
    *pFileSize =::GetFileSize( hFile, NULL ); 
    if(*pFileSize==INVALID_FILE_SIZE) 
    {
        bRet=FALSE;
    }

    CloseHandle(hFile);
    return bRet; 
}

//-----------------------------------------------------------------------------
// Name: GetFileName
// Object: Get file name from FullPath
// Parameters :
//     in : TCHAR* FullPath : full path
// Return : pointer on FileName inside FullPath buffer, NULL on error
//-----------------------------------------------------------------------------
TCHAR* CStdFileOperations::GetFileName(TCHAR* FullPath)
{
    if (IsBadReadPtr(FullPath,sizeof(TCHAR)))
        return NULL;

    TCHAR* psz;

    // find last '\'
    psz=_tcsrchr(FullPath,'\\');

    // if not found
    if (!psz)
        return FullPath;

    // go after '\'
    psz++;

    return psz;
}

//-----------------------------------------------------------------------------
// Name: GetFilePath
// Object: Get Path from FullPath keeps last '\'
// Parameters :
//     in : TCHAR* FullPath : full path
//          DWORD PathMaxSize : path max size in TCHAR
//          TCHAR* Path : app path with ending '\'
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetFilePath(IN TCHAR* FullPath,OUT TCHAR* Path,IN DWORD PathMaxSize)
{
    if (IsBadReadPtr(FullPath,sizeof(TCHAR)))
        return FALSE;

    if (IsBadWritePtr(Path,sizeof(TCHAR)))
        return FALSE;

    TCHAR* psz;
    // don't put *Path=0; in case FullPath and Path point to the same buffer
    // *Path=0;

    // find last '\'
    psz=_tcsrchr(FullPath,'\\');

    // if not found
    if (!psz)
    {
        *Path=0;
        return FALSE;
    }

    // we want to keep '\'
    psz++;
    if (PathMaxSize<(DWORD)(psz-FullPath))
    {
        *Path=0;
        return FALSE;
    }

    _tcsncpy(Path,FullPath,psz-FullPath);
    Path[psz-FullPath]=0;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: ChangeFileExt
// Object: replace old extension of FileName by the new provided one
// Parameters :
//     in : TCHAR* FileName : filename, buffer should long enough to support new extension
//          TCHAR* NewExtension : new extension without '.'
// Return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::ChangeFileExt(IN OUT TCHAR* FileName,IN TCHAR* NewExtension)
{
    if (IsBadReadPtr(NewExtension,sizeof(TCHAR)))
        return FALSE;

    TCHAR* psz=CStdFileOperations::GetFileExt(FileName);
    if (*psz==0)
    {
        // no extension
        _tcscat(FileName,_T("."));
        _tcscat(FileName,NewExtension);
        return TRUE;
    }
    if (IsBadWritePtr(psz,(_tcslen(NewExtension)+1)*sizeof(TCHAR)))
        return FALSE;

    _tcscpy(psz,NewExtension);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: RemoveFileExt
// Object: remove extension of FileName
// Parameters :
//     in : TCHAR* FileName : filename
// Return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::RemoveFileExt(IN OUT TCHAR* FileName)
{
    
    TCHAR* psz=CStdFileOperations::GetFileExt(FileName);
    if (*psz==0)// protect us against our empty const string which is in non writable section
        return FALSE;
    
    // go back to '.' as GetFileExt gives us a pointer after '.'
    psz--;
    *psz=0;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetFileDrive
// Object: get file drive
// Parameters :
//     in : TCHAR* FileName : filename
//     out :TCHAR* FileDrive : must be _MAX_DRIVE len at least
//
// Return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetFileDrive(IN TCHAR* FilePath,OUT TCHAR* FileDrive)
{
    // _tsplitpath can be used too
    if (IsBadReadPtr(FilePath,sizeof(TCHAR)))
        return NULL;

    if (IsBadWritePtr(FileDrive,sizeof(TCHAR)*_MAX_DRIVE))
        return NULL;

    _tsplitpath(FilePath,FileDrive,NULL,NULL,NULL);
    return (*FileDrive!=0);
}


//-----------------------------------------------------------------------------
// Name: GetFileExt
// Object: get pointer on file extension (point after last '.' or _T("") if no '.' found)
// Parameters :
//     in : TCHAR* FileName : filename
// Return : pointer on file ext in FileName buffer, _T("") on error
//-----------------------------------------------------------------------------
TCHAR* CStdFileOperations::GetFileExt(IN TCHAR* FileName)
{
    // _tsplitpath can be used too
    if (IsBadReadPtr(FileName,sizeof(TCHAR)))
        return NULL;

    TCHAR* psz;
    TCHAR* psz2;
    // _tcsrtok
    
    // find last point
    psz=_tcsrchr(FileName,'.');
    psz2=_tcsrchr(FileName,'\\');

    // if not found
    if ( (!psz) 
         || (psz2>psz) // file with no ext
       )
        return _T("");
    // else

    // go after point
    psz++;
    
    return psz;
}

//-----------------------------------------------------------------------------
// Name: DoesExtensionMatch
// Object: check if filename has the same extension as the provided one
// Parameters :
//     in : TCHAR* FileName : filename
//          TCHAR* Extension : extension to check (without "." like "exe", "zip", ...)
// Return : TRUE if extension match
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::DoesExtensionMatch(IN TCHAR* FileName,IN TCHAR* Extension)
{
    if (IsBadReadPtr(Extension,sizeof(TCHAR)))
        return NULL;

    TCHAR* psz;
    
    // get extension
    psz=CStdFileOperations::GetFileExt(FileName);
    if(*psz==0)
        return FALSE;

    // check extension
    return (_tcsicmp(psz,Extension)==0);
}

//-----------------------------------------------------------------------------
// Name: GetAppName
// Object: get current application name full path
// Parameters :
//     in : DWORD ApplicationNameMaxSize : user allocated buffer max size
//     in out : TCHAR* ApplicationName : fill full path
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetAppName(OUT TCHAR* ApplicationName,IN DWORD ApplicationNameMaxSize)
{
    *ApplicationName=0;
    return ::GetModuleFileName(::GetModuleHandle(NULL),ApplicationName,ApplicationNameMaxSize);
}

//-----------------------------------------------------------------------------
// Name: GetAppPath
// Object: get current application path (keep last '\')
// Parameters :
//     in : DWORD ApplicationPathMaxSize : user allocated buffer max size
//     in out : TCHAR* ApplicationPath : fill a user allocated buffer
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetAppPath(OUT TCHAR* ApplicationPath,IN DWORD ApplicationPathMaxSize)
{
    if (!CStdFileOperations::GetAppName(ApplicationPath,ApplicationPathMaxSize))
        return FALSE;
    return CStdFileOperations::GetFilePath(ApplicationPath,ApplicationPath,ApplicationPathMaxSize);
}

//-----------------------------------------------------------------------------
// Name: GetModulePath
// Object: get specified module path (keep last '\')
// Parameters :
//     in : HMODULE hModule : Module handle
//          DWORD ModulePathMaxSize : user allocated buffer max size
//     in out : TCHAR* ModulePath : fill a user allocated buffer
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetModulePath(IN HMODULE hModule,OUT TCHAR* ModulePath,DWORD ModulePathMaxSize)
{
    // get module file name
    DWORD dwRes=::GetModuleFileName(hModule,ModulePath,ModulePathMaxSize);
    // in case of failure of if buffer is too small
    if ((dwRes==0)||(dwRes==ModulePathMaxSize))
        return FALSE;

    return CStdFileOperations::GetFilePath(ModulePath,ModulePath,ModulePathMaxSize);
}

//-----------------------------------------------------------------------------
// Name: IsFullPath
// Object: Check if filename contains full path
// Parameters :
//     in : TCHAR* FileName : file name to check 
// Return : TRUE if contains full path, FALSE if FileName contains no path info (filename only)
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::IsFullPath(TCHAR* FileName)
{
    // look for a backslash
    return (_tcschr(FileName,'\\')!=0);
}

//-----------------------------------------------------------------------------
// Name: GetTempFileName
// Object: get temporary file name in the directory of BaseFileName
// Parameters :
//     in : TCHAR* BaseFileName : temporary file wil be created in the same directory as BaseFileName
//     in out : TCHAR* TempFileName: This buffer should be MAX_PATH characters
//                  to accommodate the path plus the terminating null character
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetTempFileName(IN TCHAR* BaseFileName,OUT TCHAR* TempFileName)
{
    TCHAR Path[MAX_PATH];
    if (!CStdFileOperations::GetFilePath(BaseFileName,Path,MAX_PATH))
        return FALSE;
    return CStdFileOperations::GetTempFileNameInsideDirectory(Path,TempFileName);
}
//-----------------------------------------------------------------------------
// Name: GetTempFileNameInsideDirectory
// Object: get temporary file name in the directory specified
// Parameters :
//     in : TCHAR* BaseFileName : name used to create temp file
//     in out : TCHAR* TempFileName: This buffer should be MAX_PATH characters
//                  to accommodate the path plus the terminating null character
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetTempFileNameInsideDirectory(IN TCHAR* Directory,OUT TCHAR* TempFileName)
{
    return (::GetTempFileName(Directory,_T("~"),0,TempFileName)!=0);
}
//-----------------------------------------------------------------------------
// Name: GetTempFileName
// Object: get temporary file name in windows temporary directory
// Parameters :
//     in out : TCHAR* TempFileName: This buffer should be MAX_PATH characters
//                  to accommodate the path plus the terminating null character
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetTempFileName(OUT TCHAR* TempFileName)
{
    TCHAR Path[MAX_PATH];
    ::GetTempPath(MAX_PATH,Path);
    return CStdFileOperations::GetTempFileNameInsideDirectory(Path,TempFileName);
}

//-----------------------------------------------------------------------------
// Name: CreateDirectory
// Object: create directory with all intermediate directories 
// Parameters :
//     in : TCHAR* Directory : Directory to create
// Return : TRUE on success or if directory was already existing
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::CreateDirectory(TCHAR* Directory)
{
    DWORD dwLastError;
    size_t Len=_tcslen(Directory);
    // remove ending back slash if any
    if (Directory[Len-1]=='\\')
        Directory[Len-1]=0;

    // create directory
    if (::CreateDirectory(Directory,NULL)!=0)
        return TRUE;

    // on failure
    // get last error
    dwLastError=GetLastError();
    switch(dwLastError)
    {
    case ERROR_ALREADY_EXISTS:
        // directory already exists --> nothing to do
        return TRUE;
    case ERROR_PATH_NOT_FOUND:
        {
            // upper dir doesn't exists --> create it

            // get upper dir
            TCHAR* psz;
            TCHAR* UpperDir=(TCHAR*)_alloca((_tcslen(Directory)+1)*sizeof(TCHAR));
            _tcscpy(UpperDir,Directory);
            // find last '\\'
            psz=_tcsrchr(UpperDir,'\\');
            if (!psz)
                return FALSE;
            // ends upper dir
            *psz=0;

             if (!CStdFileOperations::CreateDirectory(UpperDir))
                 return FALSE;

             return (::CreateDirectory(Directory,NULL)!=0);
        }
    default:
        return FALSE;
    }
}

//-----------------------------------------------------------------------------
// Name: CreateDirectoryForFile
// Object: create directory if directory does not exists for specified FileName
//         can be used before calling CreateFile, to assume directory exists and avoid CreateFile failure
// Parameters :
//     in : TCHAR* FileNameToCreate : name of file that is going to be created
// Return : TRUE on success or if directory was already existing
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::CreateDirectoryForFile(TCHAR* FileNameToCreate)
{
    TCHAR szPath[MAX_PATH];
    // get path
    CStdFileOperations::GetFilePath(FileNameToCreate,szPath,MAX_PATH);
    // check if directory already exists
    if (CStdFileOperations::DoesDirectoryExists(szPath))
        return TRUE;
    // create directory
    return CStdFileOperations::CreateDirectory(szPath);
}

//-----------------------------------------------------------------------------
// Name: DoesDirectoryExists
// Object: check if directory exists
// Parameters :
//     in : TCHAR* Directory : directory name
// Return : TRUE if directory exists
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::DoesDirectoryExists(TCHAR* Directory)
{
    BOOL bFileExists=FALSE;
    HANDLE hFile=CreateFile (
                            Directory,
                            GENERIC_READ,
                            FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
                            NULL,
                            OPEN_EXISTING,
                            FILE_FLAG_BACKUP_SEMANTICS,
                            NULL
                            );
    if (hFile != INVALID_HANDLE_VALUE)
    {
        bFileExists=TRUE;
        CloseHandle(hFile);
    }
    return bFileExists;
}

//-----------------------------------------------------------------------------
// Name: IsDirectory
// Object: assume FullPath exists and is a directory
// Parameters :
//     in : TCHAR* Directory : directory name
// Return : TRUE if directory exists
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::IsDirectory(TCHAR* FullPath)
{
    DWORD Ret;
    Ret=::GetFileAttributes(FullPath);
    if (Ret==INVALID_FILE_ATTRIBUTES)
        return FALSE;

    return (Ret & FILE_ATTRIBUTE_DIRECTORY);
}

//-----------------------------------------------------------------------------
// Name: GetAbsolutePath
// Object: get absolute path from relative path
// Parameters :
//     in : TCHAR* RelativePath : Relative path
//     out : TCHAR* AbsolutePath : absolute path must be at least MAX_PATH len in TCHAR
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetAbsolutePath(TCHAR* RelativePath,OUT TCHAR* AbsolutePath)
{
    return CStdFileOperations::GetAbsolutePath(NULL,RelativePath,AbsolutePath);
}

//-----------------------------------------------------------------------------
// Name: GetAbsolutePath
// Object: get absolute path from relative path
// Parameters :
//     in :  TCHAR* ReferenceDirectory : if NULL the current directory will be used
//           TCHAR* RelativePath : Relative path
//     out : TCHAR* AbsolutePath : absolute path must be at least MAX_PATH len in TCHAR
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetAbsolutePath(TCHAR* ReferenceDirectory,TCHAR* RelativePath,OUT TCHAR* AbsolutePath)
{
    if (IsBadReadPtr(RelativePath,sizeof(TCHAR)))
        return NULL;

    if (IsBadWritePtr(AbsolutePath,sizeof(TCHAR)))
        return NULL;

    LPTSTR pszFileName = NULL;
    if (AbsolutePath)
        *AbsolutePath=0;
    if (RelativePath==NULL)
        return FALSE;
#ifdef _DEBUG
    if (RelativePath == AbsolutePath)
    {
        // Relative path must differ from absolute path
        DebugBreak();
    }
#endif

    TCHAR CurrentDirectory[MAX_PATH];
    // save current directory
    ::GetCurrentDirectory(_countof(CurrentDirectory),CurrentDirectory);
    if (ReferenceDirectory)
    {
        // adjust current directory for GetFullPathName
        ::SetCurrentDirectory(ReferenceDirectory);
    }
    else
    {   
        TCHAR ApplicationDirectory[MAX_PATH];
        // we need to force directory to application directory (else CurrentDirectory can depends of process who launched the application, or of user browsing)
        CStdFileOperations::GetAppPath(ApplicationDirectory,MAX_PATH);

        // adjust current directory for GetFullPathName
        ::SetCurrentDirectory(ApplicationDirectory);
    }

    // _tfullpath can be used too for more portable way (in this file we call as much API as we can to be linked only to kernel32)
    BOOL bRet = (::GetFullPathName(RelativePath,MAX_PATH,AbsolutePath,&pszFileName)!=0);

    // restore current directory
    ::SetCurrentDirectory(CurrentDirectory);

    return bRet;
}


//-----------------------------------------------------------------------------
// Name: ContainsInvalidPathChar
// Object: check if string contains invalid path chars
// Parameters :
//     in : TCHAR* Str : string to be tested
// Return : TRUE if string contains invalid chars
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::ContainsInvalidPathChar(TCHAR* Str)
{
    TCHAR* psz = _tcspbrk(Str,_T("\\/:*?|<>"));
    return (psz!=NULL);
}

//-----------------------------------------------------------------------------
// Name: GetRelativePath
// Object: create relative path from one location to another location
// Parameters :
//      IN TCHAR* From : Original directory
//      IN TCHAR* To : Destination directory
//      OUT TCHAR* RelativePath : Relative path from "From" to "To"
// Return : TRUE if a relative path can be made, FALSE if it can't be done (in this case RelativePath is a copy of To)
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetRelativePath(IN TCHAR* From,IN TCHAR* To,OUT TCHAR* RelativePath)
{
    // assume From and To doesn't end with '\\'
    TCHAR* LocalFrom;
    TCHAR* LocalTo;
    SIZE_T Size;

    Size = _tcslen(From);
    if (Size==0)
    {
        // no relative path can be made
        // copy full target path
        _tcscpy(RelativePath,To);
        return FALSE;
    }
    LocalFrom = new TCHAR[Size+2];// +2 : 1 for \0 and 1 for potential '\\'
    _tcscpy(LocalFrom,From);
    if (LocalFrom[Size-1]!='\\')
    {
        LocalFrom[Size]='\\';
        LocalFrom[Size+1]=0;
    }

    Size = _tcslen(To);
    if (Size==0)
    {
        // no relative path can be made
        // copy full target path
        _tcscpy(RelativePath,To);
        delete[] LocalFrom;
        return FALSE;
    }
    LocalTo = new TCHAR[Size+2];// +2 : 1 for \0 and 1 for potential '\\'
    _tcscpy(LocalTo,To);
    if (LocalTo[Size-1]!='\\')
    {
        LocalTo[Size]='\\';
        LocalTo[Size+1]=0;
    }

    TCHAR* pszSplitterFrom;
    TCHAR* pszSplitterTo;
    TCHAR* pszPreviousSplitterFrom = LocalFrom;
    TCHAR* pszPreviousSplitterTo = LocalTo;
    TCHAR DriveFrom[MAX_PATH];
    TCHAR DriveTo[MAX_PATH];

    // if same path
    if (_tcsicmp(LocalFrom,LocalTo)==0)
    {
        _tcscpy(RelativePath,_T(".\\"));
        delete[] LocalFrom;
        delete[] LocalTo;
        return TRUE;
    }

    // get drives
    CStdFileOperations::GetFileDrive(LocalFrom,DriveFrom);
    CStdFileOperations::GetFileDrive(LocalTo,DriveTo);

    // if not on the same drive
    if (_tcsicmp(DriveFrom,DriveTo)!=0)
    {
        // no relative path can be made
        // copy full target path
        _tcscpy(RelativePath,LocalTo);
        delete[] LocalFrom;
        delete[] LocalTo;
        return FALSE;
    }

    // find common part
    for(;;)
    {
        pszSplitterFrom = _tcschr(pszPreviousSplitterFrom+1,'\\');
        pszSplitterTo = _tcschr(pszPreviousSplitterTo+1,'\\');
        if (!pszSplitterFrom)
            break;
        if (!pszSplitterTo)
            break;
        if(_tcsnicmp(LocalFrom,LocalTo,pszSplitterFrom-LocalFrom)!=0)
            break;
        pszPreviousSplitterFrom = pszSplitterFrom;
        pszPreviousSplitterTo = pszSplitterTo;
    }

    // get the upper directories
    *RelativePath =0;
    for(;;)
    {
        pszSplitterFrom = _tcschr(pszPreviousSplitterFrom+1,'\\');
        if (!pszSplitterFrom)
            break;
        pszPreviousSplitterFrom = pszSplitterFrom;
        _tcscat(RelativePath,_T("..\\"));
    }
    // add relative part
    _tcscat(RelativePath,pszPreviousSplitterTo+1);
    delete[] LocalFrom;
    delete[] LocalTo;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: IsFileInDirectoryOrSubDirectory
// Object: Check if "File" belongs to directory or subdirectory of "Directory"
//         Notice : Do not check if file exists ! Use the CStdFileOperations::DoesFileExists in this case
//                  Checking is only done on paths string
// Parameters :
//      IN TCHAR* File : file to check
//      IN TCHAR* Directory : root of directories to check (can ends with '\\' or not)
// Return : TRUE if "File" belongs to directory or subdirectory of "Directory"
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::IsFileInDirectoryOrSubDirectory(IN TCHAR* File,IN TCHAR* Directory)
{
    // only check if begin of path is the same
    SIZE_T DirectorySize = _tcslen(Directory);
    if (_tcsnicmp(File,Directory,DirectorySize) != 0)
        return FALSE;
    // if directory ends with '\\'
    if (Directory[DirectorySize-1]=='\\')
        DirectorySize--; // we must access char at (DirectorySize-1) in File name for checking
    if (File[DirectorySize]!='\\') // assume file has not directory prefix followed other alphanumerics chars 
        return FALSE;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: IsRelativePath
// Object: check if path is relative (avoid to include extra shell lib)
// Parameters :
//      IN TCHAR* File : Path
// Return : TRUE if relative path
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::IsRelativePath(IN TCHAR* Path)
{
    TCHAR FirstChar;
    if (Path==NULL)
        return FALSE;

    FirstChar = *Path;

    if (FirstChar == 0)
        return FALSE;
#if (defined(UNICODE)||defined(_UNICODE))

#else
    if (::IsDBCSLeadByte(*Path))
        return FALSE;
#endif

    if (FirstChar=='\\')
    {
#if (defined(UNICODE)||defined(_UNICODE))
        //if (_tcsncmp(Path,TEXT("\\\\?\\"),4) == 0 )
        //    return FALSE;
#endif
        return FALSE;
    }

    if (Path[1] != ':')
        return TRUE;

    BOOL IsDriverLetter =  ( ( FirstChar>='a' ) && ( FirstChar<='z' ) ) ||( ( FirstChar>='A' ) && ( FirstChar<='Z' ) );
    return !IsDriverLetter;
}
//-----------------------------------------------------------------------------
// Name: GetUpperDirectory
// Object: Notice : provided path is directly affected
// Parameters :
//      IN OUT TCHAR* File : in : Path, out : upper directory
// Return : TRUE if upper directory exists
//-----------------------------------------------------------------------------
BOOL CStdFileOperations::GetUpperDirectory(IN OUT TCHAR* Path)
{
    TCHAR* psz;

    if (Path == NULL)
        return FALSE;
    if (*Path == 0)
        return FALSE;

    // if path is a directory, remove ending '\\' if any
    SIZE_T PathSize = _tcslen(Path);
    if (Path[PathSize-1] == '\\')
        Path[PathSize-1] = 0;

    // get upper dir
    psz=_tcsrchr(Path,'\\');
    if (psz==NULL)
        return FALSE;

    // if '\\' is first char
    if (psz == Path)
        return FALSE;

    // check for network drive begin
    if (*(psz-1) == '\\')
        return FALSE;

#if (defined(UNICODE)||defined(_UNICODE))
    // check for long filenames
    // aka check for directories beginning with TEXT("\\\\?\\")
    if (*(psz-1) == '?')
        return FALSE;
#endif

    // ends Path so it contains upper directory
    *psz = 0;

    return TRUE;
}